// (C) 2004 D. Gauchard <deyv@free.fr>
// Licensed under the terms of the GNU Public License version 2

#include "topheader.h"

#define LEN	128
#define BIGLEN	1024

#if defined(__CYGWIN32) || defined(_WIN32)
#define snprintf(a,b,c...) sprintf(a,c)
#endif

void help (char* name)
{
	printf("Syntax: %s <file-name> <struct-name> <filename> [<filename>...]\n", name);
}

void build_defname (char* defname, char* filename)
{
	int i;
	char c;
	strcpy(defname, filename);
	i = 0;
	do
	{
		switch (c = filename[i])
		{
			case '.':
			case '-':
			case '/':
				c = '_';
				break;
		}
		defname[i++] = c;
	} while (c);
}
		
void build_defnamelen (char* defnamelen, char* filename)
{
	int i;
	char c;
	strcpy(defnamelen, filename);
	i = 0;
	do
	{
		switch (c = toupper(filename[i]))
		{
			case '.':
			case '-':
			case '/':
				c = '_';
				break;
		}
		defnamelen[i++] = c;
	} while (c);
	strcat(defnamelen, "_LEN");
}
		
int main (int argc, char* argv[])
{
	FILE* f;
	FILE* fc;
	FILE* fh;
	char fcname[LEN], fhname[LEN], temp[LEN], decl[BIGLEN];
	char* filename;
	char* headername;
	off_t size, i;
	unsigned char* buf;
	int nf;
	
	if (argc < 3)
	{
		help(argv[0]);
		return 1;
	}
	
	filename = argv[1];
	headername = argv[2];
	
	snprintf(fhname, LEN, "%s.h", filename);
	
	/*
	 * fopen failing in update means the file may not exist.
	 * Write file banner in that case.
	 */
	if ((fh = fopen(fhname, "r+")) == NULL)
	{
		if (errno != ENOENT)
		{
			perror(fhname);
			return 1;
		}
		if ((fh = fopen(fhname, "w")) == NULL)
		{
			perror(fhname);
			return 1;
		}
		fprintf(fh, "/* This file is automatically generated */\n"
			    "\n"
			    "#ifndef __BINWARE_H__\n"
			    "#define __BINWARE_H__\n"
			    "\n"
			    "typedef struct\n"
			    "{\n"
			    "\tunsigned char*\tname;\n"
			    "\tint\t\tsize;\n"
			    "\tunsigned char*\tdata;\n"
			    "} binware_s;\n"
			    "\n");
	}
	else if (fseek(fh, 0, SEEK_END) == -1)
	{
		perror(fhname);
		return 1;
	}
	
	snprintf(fcname, LEN, "%s.c", filename);
	if ((fc = fopen(fcname, "r+")) == NULL)
	{
		if (errno != ENOENT)
		{
			perror(fcname);
			return 1;
		}
		if ((fc = fopen(fcname, "w")) == NULL)
		{
			perror(fcname);
			return 1;
		}
		fprintf(fc, "/* This file is automatically generated */\n"
			    "\n"
			    "#include <stdlib.h>\n"
			    "#include \"%s\"\n"
			    "\n",
			    fhname);
	}
	else if (fseek(fc, 0, SEEK_END) == -1)
	{
		perror(fcname);
		return 1;
	}
	
	fprintf(fh, "extern binware_s binware_%s [];\n", headername);
	snprintf(decl, BIGLEN, "binware_s binware_%s [] =\n{", headername);

	for (nf = 3; nf < argc; nf++)
	{
		int ret;
		struct stat st;
		char defname[LEN];
		char defnamelen[LEN];
		build_defname(defname, argv[nf]);
		build_defnamelen(defnamelen, argv[nf]);

		if (stat(argv[nf], &st) == -1)
		{
			perror("stat");
			return 1;
		}
		size = st.st_size;

		if ((f = fopen(argv[nf], "rb")) == NULL)
		{
			perror(argv[nf]);
			return 1;
		}

		if ((buf = (unsigned char*)malloc(size)) == NULL)
		{
			perror("malloc");
			fprintf(stderr, "Cannot allocate %i bytes for file %s\n", (int)size, argv[nf]);
			return 1;
		}
	
		if ((ret = fread(buf, size, 1, f)) != 1)
		{
			if (ferror(f))
				perror(argv[nf]);
			fprintf(stderr, "Could not fully read file %s\n", argv[nf]);
			return 1;
		}
	
		fclose(f);
	
		fprintf(fc, "#define %s %i\n", defnamelen, (int)size);
	
		fprintf(fc, "unsigned char %s[%s] =\n{\t", defname, defnamelen);
		for (i = 0; i < size; i++)
		{
			fprintf(fc, "0x%02x, ", buf[i]);
			if ((i != (size - 1)) && !((i+1) % 16))
				fprintf(fc, "\n\t");
		}
		fprintf(fc, "\n};\n\n");
		
		free(buf);
		
		snprintf(temp, LEN, "\t{ \"%s\", %s, %s },\n", argv[nf], defnamelen, defname);
		strcat(decl, temp);
	}
	fprintf(fc, decl);
	fprintf(fc, "\t{ NULL, 0, NULL }\n};\n\n");

	fclose(fc);
	fclose(fh);

	return 0;
}
